package com.cenqua.fisheye.user.plugin;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Properties;

/**
 * An example authenticator.
 * <p>
 * This authenticator reads its data from its {@link Properties} configuration.
 * </p>
 *
 * <p>
 * Usernames/passwords are found using properties of the form <code>user.USERNAME=PASSWORD</code>
 * </p>
 *
 * <p>
 * By default, access is allowed to any repository. Access can be restricted
 * to a set of users using a property of the form
 * <code>repos.REPNAME=USERNAME1,USERNAME2,...,USERNAMEn</code>
 * </p>
 *
 * <p>To use this example, you would need something like the following in your config.xml:
 * </p>
 *
 * <pre>&lt;security allow-anon="false"&gt;
 *&lt;custom
 *    classname="com.cenqua.fisheye.user.plugin.ExampleFishEyeAuthenticator"
 *    auto-add="true"
 *    positive-cache-ttl="5mins"&gt;
 *    &lt;properties&gt;
 *user.jane=j4n3
 *user.fred=fr3d
 *repos.rep1=jane,fred
 *repos.rep2=fred
 *repos.rep3=jane
 *    &lt;/properties&gt;
 *&lt;/custom&gt;&lt;/security&gt;</pre>
 */
public class ExampleFishEyeAuthenticator extends AbstractFishEyeAuthenticator {

    public static class ExampleAuthToken implements AuthToken {
        private final String username;

        public ExampleAuthToken(String username) {
            this.username = username;
        }

        public String getUsername() {
            return username;
        }

        public String getEmail() {
            return username + "@foo.com";
        }


        public String getDisplayName() {
            // just fudge a display name, capitalise the first letter of username
            return username.substring(0, 1).toUpperCase() + username.substring(1);
        }
    }

    private Properties props;

    public void init(Properties cfg) throws Exception {
        props = cfg;
    }

    public void close() {
        System.out.println("Example: close() called");
    }

    public AuthToken checkPassword(String username, String password) {
        if ((username == null) || "".equals(username)) {
            return null;
        }

        if ((password == null) || "".equals(password)) {
            return null;
        }

        String prop = "user." + username;
        String expected = props.getProperty(prop);
        if ((expected == null) || !expected.equals(password)) {
            return null;
        }

        System.out.println("Example: found user " + username);

        return new ExampleAuthToken(username);
    }

    public AuthToken recreateAuth(String username) {
        String prop = "user." + username;
        if (props.containsKey(prop)) {
            System.out.println("Example: found user, recreating " + username);
            return new ExampleAuthToken(username);
        }
        return null;
    }

    public boolean hasPermissionToAccess(AuthToken tok, String repname, String constraint) {
        // constraint is not used in this example.

        System.out.println("Example: hasPermissionToAccess? " +
                           tok + " " + repname + " " + constraint);

        if (tok == null) {
            // anon user cannot access repos in this example.
            return false;
        }

        String prop = "repos." + repname;
        if (props.containsKey(prop)) {
            System.out.println("props = " + props);
            String[] usersList = props.getProperty(prop).split(",");
            final HashSet users = new HashSet(Arrays.asList(usersList));
            return users.contains(tok.getUsername());
        } else {
            System.out.println("No prop = " + prop + " configured. Access granted to all users.");
            return true; // open to everyone
        }
    }
    
}
